from clustaar.schemas.v1 import GOOGLE_CUSTOM_SEARCH_ACTION
from clustaar.schemas.models import GoogleCustomSearchAction
import pytest


@pytest.fixture
def action():
    return GoogleCustomSearchAction(query="Tintin", limit=1, custom_engine_id="AZERTY")


@pytest.fixture
def data():
    return {
        "type": "google_custom_search_action",
        "query": "Tintin",
        "limit": 1,
        "customEngineID": "AZERTY"
    }


class TestDump(object):
    def test_returns_a_dict(self, action, data, mapper):
        result = GOOGLE_CUSTOM_SEARCH_ACTION.dump(action, mapper)
        assert result == data


class TestLoad(object):
    def test_returns_an_action(self, data, mapper):
        action = mapper.load(data, GOOGLE_CUSTOM_SEARCH_ACTION)
        assert isinstance(action, GoogleCustomSearchAction)
        assert action.query == "Tintin"
        assert action.custom_engine_id == "AZERTY"
        assert action.limit == 1



import re
from lupin.fields import String
from lupin.fields.compatibility import merge_validator
from .validators import IsRegexp


class RegexpField(String):
    """Field storing regexps"""

    def __init__(self, **kwargs):
        merge_validator(kwargs, IsRegexp())
        super().__init__(**kwargs)

    def load(self, value, mapper):
        """Loads a regexp from JSON value

        Args:
            value (string): a pattern string
            mapper (Mapper): mapper used to load data

        Returns:
            object
        """
        if value is not None:
            return re.compile(value)

    def dump(self, value, mapper):
        """Dump a regexp

        Args:
            value (regexp): a value
            mapper (Mapper): mapper used to dump data

        Returns:
            object
        """
        if value is not None:
            return value.pattern



from clustaar.schemas.v1 import CLOSE_INTERCOM_CONVERSATION_ACTION
from clustaar.schemas.models import CloseIntercomConversationAction
import pytest


@pytest.fixture
def action():
    return CloseIntercomConversationAction()


@pytest.fixture
def data():
    return {
        "type": "close_intercom_conversation_action"
    }


class TestDump(object):
    def test_returns_a_dict(self, action, data, mapper):
        result = CLOSE_INTERCOM_CONVERSATION_ACTION.dump(action, mapper)
        assert result == data


class TestLoad(object):
    def test_returns_an_action(self, data, mapper):
        action = mapper.load(data, CLOSE_INTERCOM_CONVERSATION_ACTION)
        assert isinstance(action, CloseIntercomConversationAction)



from clustaar.schemas.v1 import PREDICATE_SESSION_VALUE_GETTER
from clustaar.schemas.models import SessionValueGetter
import pytest


@pytest.fixture
def data():
    return {
        "type": "session_value",
        "key": "name"
    }


@pytest.fixture
def getter():
    return SessionValueGetter(key="name")


class TestLoad(object):
    def test_must_return_a_getter(self, data, mapper):
        result = mapper.load(data, PREDICATE_SESSION_VALUE_GETTER)
        assert isinstance(result, SessionValueGetter)
        assert result.key == "name"


class TestDump(object):
    def test_returns_a_dict(self, getter, data, mapper):
        result = mapper.dump(getter, PREDICATE_SESSION_VALUE_GETTER)
        assert result == data



import os
import pytest


PROJECT_PATH = os.path.join(os.path.dirname(__file__), "..")
PACKAGE_PATH = os.path.join(PROJECT_PATH, "src")
os.sys.path.insert(0, PROJECT_PATH)
os.sys.path.insert(0, PACKAGE_PATH)



from clustaar.schemas.v1 import PREDICATE_MESSAGE_GETTER
from clustaar.schemas.models import MessageGetter
import pytest


@pytest.fixture
def getter():
    return MessageGetter()


@pytest.fixture
def data():
    return {
        "type": "message",
    }


class TestLoad(object):
    def test_must_return_a_getter(self, data, mapper):
        result = mapper.load(data, PREDICATE_MESSAGE_GETTER)
        assert isinstance(result, MessageGetter)


class TestDump(object):
    def test_returns_a_dict(self, getter, data, mapper):
        result = mapper.dump(getter, PREDICATE_MESSAGE_GETTER)
        assert result == data



from clustaar.schemas.v1 import PAUSE_BOT_ACTION
from clustaar.schemas.models import PauseBotAction
import pytest


@pytest.fixture
def action():
    return PauseBotAction()


@pytest.fixture
def data():
    return {
        "type": "pause_bot_action"
    }


class TestDump(object):
    def test_returns_a_dict(self, action, data, mapper):
        result = PAUSE_BOT_ACTION.dump(action, mapper)
        assert result == data


class TestLoad(object):
    def test_returns_an_action(self, data, mapper):
        action = mapper.load(data, PAUSE_BOT_ACTION)
        assert isinstance(action, PauseBotAction)



from clustaar.schemas.v1 import SEND_EMAIL_ACTION
from clustaar.schemas.models import SendEmailAction
import pytest


@pytest.fixture
def action():
    return SendEmailAction(
        from_email="tintin@gmail.com",
        from_name="Tintin",
        recipient="test@example.com",
        subject="Hello",
        content=":)"
    )


@pytest.fixture
def data():
    return {
        "type": "send_email_action",
        "fromEmail": "tintin@gmail.com",
        "fromName": "Tintin",
        "recipient": "test@example.com",
        "subject": "Hello",
        "content": ":)"
    }


class TestDump(object):
    def test_returns_a_dict(self, action, data, mapper):
        result = SEND_EMAIL_ACTION.dump(action, mapper)
        assert result == data


class TestLoad(object):
    def test_returns_an_action(self, data, mapper):
        action = mapper.load(data, SEND_EMAIL_ACTION)
        assert isinstance(action, SendEmailAction)
        assert action.content == ":)"
        assert action.subject == "Hello"
        assert action.recipient == "test@example.com"
        assert action.from_email == "tintin@gmail.com"
        assert action.from_name == "Tintin"



from clustaar.schemas.v1 import FLOW_CONNECTION
from clustaar.schemas.models import FlowConnection, IsSetCondition, MessageGetter, StepTarget,\
    ConnectionPredicate
import pytest


@pytest.fixture
def data():
    return {
        "type": "flow_connection",
        "target": {
            "id": "a1" * 12,
            "type": "step",
            "name": "a step"
        },
        "predicates": [
            {
                "type": "connection_predicate",
                "condition": {
                    "type": "is_set"
                },
                "valueGetter": {
                    "type": "message"
                }
            }
        ]
    }


@pytest.fixture
def connection():
    predicate = ConnectionPredicate(
        condition=IsSetCondition(),
        value_getter=MessageGetter()
    )
    return FlowConnection(
        predicates=[predicate],
        target=StepTarget(step_id="a1" * 12, name="a step")
    )


class TestLoad(object):
    def test_returns_a_predicate(self, data, mapper):
        result = mapper.load(data, FLOW_CONNECTION)
        assert isinstance(result, FlowConnection)
        target = result.target
        assert isinstance(target, StepTarget)
        assert target.step_id == "a1" * 12
        predicate = result.predicates[0]
        assert isinstance(predicate, ConnectionPredicate)
        assert isinstance(predicate.condition, IsSetCondition)
        assert isinstance(predicate.value_getter, MessageGetter)


class TestDump(object):
    def test_returns_a_dict(self, data, connection, mapper):
        result = mapper.dump(connection, FLOW_CONNECTION)
        assert result == data



from clustaar.schemas.v1 import STORE_SESSION_VALUE_ACTION
from clustaar.schemas.models import StoreSessionValueAction
import pytest


@pytest.fixture
def action():
    return StoreSessionValueAction(key="var1", value="val1")


@pytest.fixture
def data():
    return {
        "type": "store_session_value_action",
        "key": "var1",
        "value": "val1"
    }


class TestDump(object):
    def test_returns_a_dict(self, action, data, mapper):
        result = STORE_SESSION_VALUE_ACTION.dump(action, mapper)
        assert result == data


class TestLoad(object):
    def test_returns_an_action(self, data, mapper):
        action = mapper.load(data, STORE_SESSION_VALUE_ACTION)
        assert isinstance(action, StoreSessionValueAction)
        assert action.key == "var1"
        assert action.value == "val1"



from clustaar.schemas.v1 import IS_NOT_SET_CONDITION
from clustaar.schemas.models import IsNotSetCondition
import pytest


@pytest.fixture
def data():
    return {
        "type": "is_not_set"
    }


@pytest.fixture
def condition():
    return IsNotSetCondition()


class TestLoad(object):
    def test_must_return_a_condition(self, data, mapper):
        result = mapper.load(data, IS_NOT_SET_CONDITION)
        assert isinstance(result, IsNotSetCondition)


class TestDump(object):
    def test_returns_a_dict(self, condition, data, mapper):
        result = mapper.dump(condition, IS_NOT_SET_CONDITION)
        assert result == data



from clustaar.schemas.validators import ObjectID
from lupin.errors import InvalidMatch
import pytest


@pytest.fixture
def validator():
    return ObjectID()


class TestCall(object):
    def test_raise_error_if_invalid_oid(self, validator):
        with pytest.raises(InvalidMatch):
            validator("", [])

    def test_does_not_raise_error_if_valid_oid(self, validator):
        validator("a1" * 12, [])



from clustaar.schemas.v1 import ASSIGN_INTERCOM_CONVERSATION_ACTION
from clustaar.schemas.models import AssignIntercomConversationAction
import pytest


@pytest.fixture
def action():
    return AssignIntercomConversationAction(assignee_id="FJNBDDHH654789")


@pytest.fixture
def data():
    return {
        "type": "assign_intercom_conversation_action",
        "assigneeID": "FJNBDDHH654789"
    }


class TestDump(object):
    def test_returns_a_dict(self, action, data, mapper):
        result = ASSIGN_INTERCOM_CONVERSATION_ACTION.dump(action, mapper)
        assert result == data


class TestLoad(object):
    def test_returns_an_action(self, data, mapper):
        action = mapper.load(data, ASSIGN_INTERCOM_CONVERSATION_ACTION)
        assert isinstance(action, AssignIntercomConversationAction)
        assert action.assignee_id == "FJNBDDHH654789"



from clustaar.schemas.v1 import FLOW_CONNECTION_PREDICATE
from clustaar.schemas.models import ConnectionPredicate, IsSetCondition, MessageGetter
import pytest


@pytest.fixture
def data():
    return {
        "type": "connection_predicate",
        "condition": {
            "type": "is_set"
        },
        "valueGetter": {
            "type": "message"
        }
    }


@pytest.fixture
def predicate():
    return ConnectionPredicate(
        condition=IsSetCondition(),
        value_getter=MessageGetter()
    )


class TestLoad(object):
    def test_returns_a_predicate(self, data, mapper):
        result = mapper.load(data, FLOW_CONNECTION_PREDICATE)
        assert isinstance(result, ConnectionPredicate)
        assert isinstance(result.condition, IsSetCondition)
        assert isinstance(result.value_getter, MessageGetter)


class TestDump(object):
    def test_returns_a_dict(self, data, mapper, predicate):
        result = mapper.dump(predicate, FLOW_CONNECTION_PREDICATE)
        assert result == data



import pytest

from clustaar.schemas.models import CreateZendeskTicketAction, ZendeskUser
from clustaar.schemas.v1 import CREATE_ZENDESK_TICKET_ACTION
from lupin.errors import InvalidDocument, InvalidLength, NotEqual, InvalidMatch
from clustaar.schemas.constants import (
    ZENDESK_TICKET_TYPES,
    ZENDESK_TICKET_PRIORITIES,
    ZENDESK_TICKET_TAG_MAX_LENGTH,
    ZENDESK_TICKET_TAGS_MAX_COUNT,
    ZENDESK_USER_NAME_MAX_LENGTH,
    ZENDESK_USER_EMAIL_MAX_LENGTH,
    ZENDESK_TICKET_SUBJECT_MAX_LENGTH,
    ZENDESK_TICKET_GROUP_ID_MAX_LENGTH,
    ZENDESK_TICKET_ASSIGNEE_ID_MAX_LENGTH,
    ZENDESK_TICKET_DESCRIPTION_MAX_LENGTH,
)


@pytest.fixture
def action(user):
    return CreateZendeskTicketAction(
        group_id="12" * 12,
        assignee_id="21" * 12,
        subject="Tester cette action",
        description="Pfff aucune idée",
        tags=["finished", "fish", "turtle"],
        ticket_type=list(ZENDESK_TICKET_TYPES)[0],
        ticket_priority=list(ZENDESK_TICKET_PRIORITIES)[0],
        user=ZendeskUser(
            email="Tintin@doe.fifi",
            name="Je suis un super test",
            phone_number="0611654852"
        )
    )


@pytest.fixture
def user():
    return {
        "name": "Je suis un super test",
        "email": "Tintin@doe.fifi",
        "phoneNumber": "0611654852"
    }


@pytest.fixture
def data(user):
    return {
        "ticketPriority": list(ZENDESK_TICKET_PRIORITIES)[0],
        "ticketType": list(ZENDESK_TICKET_TYPES)[0],
        "type": "create_zendesk_ticket_action",
        "tags": ["finished", "fish", "turtle"],
        "description": "Pfff aucune idée",
        "subject": "Tester cette action",
        "assigneeID": "21" * 12,
        "groupID": "12" * 12,
        "user": user
    }


def assert_raise_on_length(mapper, data):
    with pytest.raises(InvalidDocument) as errors:
        mapper.validate(data, CREATE_ZENDESK_TICKET_ACTION)

    error = errors.value[0]
    assert isinstance(error, InvalidLength)


def assert_raise_on_equal(mapper, data):
    with pytest.raises(InvalidDocument) as errors:
        mapper.validate(data, CREATE_ZENDESK_TICKET_ACTION)

    error = errors.value[0]
    assert isinstance(error, NotEqual)


def assert_raise_on_format(mapper, data):
    with pytest.raises(InvalidDocument) as errors:
        mapper.validate(data, CREATE_ZENDESK_TICKET_ACTION)

    error = errors.value[0]
    assert isinstance(error, InvalidMatch)


class TestDump(object):
    def test_returns_a_dict(self, action, data, mapper):
        result = CREATE_ZENDESK_TICKET_ACTION.dump(action, mapper)
        assert result == data


class TestLoad(object):
    def test_returns_an_action(self, data, mapper):
        action = mapper.load(data, CREATE_ZENDESK_TICKET_ACTION)
        assert isinstance(action, CreateZendeskTicketAction)


class TestValidate(object):
    def test_raise_if_dirty_ticket_type(self, mapper, data, user):
        data["ticketType"] = "hh"
        assert_raise_on_equal(mapper, data)

    def test_raise_if_dirty_ticket_priority(self, mapper, data, user):
        data["ticketPriority"] = "hh"
        assert_raise_on_equal(mapper, data)

    def test_raise_if_dirty_name(self, mapper, data, user):
        user["name"] = "a" * (ZENDESK_USER_NAME_MAX_LENGTH + 1)
        assert_raise_on_length(mapper, data)

    def test_raise_if_dirty_email(self, mapper, data, user):
        user["email"] = "a" * (ZENDESK_USER_EMAIL_MAX_LENGTH + 1)
        assert_raise_on_length(mapper, data)

    def test_raise_if_dirty_subject(self, mapper, data):
        data["subject"] = "a" * (ZENDESK_TICKET_SUBJECT_MAX_LENGTH + 1)
        assert_raise_on_length(mapper, data)

    def test_raise_if_dirty_description(self, mapper, data):
        data["description"] = "a" * (ZENDESK_TICKET_DESCRIPTION_MAX_LENGTH + 1)
        assert_raise_on_length(mapper, data)

        data["description"] = ""
        assert_raise_on_length(mapper, data)

    def test_raise_if_dirty_group_id(self, mapper, data):
        data["groupID"] = "1" * (ZENDESK_TICKET_GROUP_ID_MAX_LENGTH + 1)
        assert_raise_on_length(mapper, data)

        data["groupID"] = "1j"
        assert_raise_on_format(mapper, data)

    def test_raise_if_dirty_assignee_id(self, mapper, data):
        data["assigneeID"] = "1" * (ZENDESK_TICKET_ASSIGNEE_ID_MAX_LENGTH + 1)
        assert_raise_on_length(mapper, data)

        data["assigneeID"] = "1j"
        assert_raise_on_format(mapper, data)

    def test_raise_if_dirty_tags(self, mapper, data):
        data["tags"] = [str(n) for n in range(
            (ZENDESK_TICKET_TAGS_MAX_COUNT + 1))]
        assert_raise_on_length(mapper, data)

        data["tags"] = ["a" * (ZENDESK_TICKET_TAG_MAX_LENGTH + 1)]
        assert_raise_on_length(mapper, data)
